#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/init.h>

MODULE_LICENSE("Public Domain");

/* do nothing */
struct inode_operations emptyfs_iops = { NULL, };
struct file_operations  emptyfs_fops = { NULL, };

/* This is called at mount time */
struct inode *emptyfs_get_inode(struct super_block *sb, int mode, dev_t dev)
{
        struct inode * inode;

	/* only succeed for the root dir */
	if ((mode & S_IFMT) != S_IFDIR) return NULL;
	if (!(inode = new_inode(sb))) return NULL;
	inode->i_mode = mode;
	inode->i_op = &emptyfs_iops;
	inode->i_fop = &emptyfs_fops;
	inode->i_nlink++; /* dirs have nlink >= 2 minimum */
        return inode;
}

struct super_operations emptyfs_ops = { /* use kernel helpers */
        .statfs         = simple_statfs,
        .drop_inode     = generic_delete_inode,
};

int emptyfs_fill_super(struct super_block * sb, void * data, int silent)
{
        struct inode * inode;
        struct dentry * root;

        sb->s_op = &emptyfs_ops;
        inode = emptyfs_get_inode(sb, S_IFDIR | 0755, 0);
        root = d_alloc_root(inode);
        if (!root) {
                iput(inode);
                return -ENOMEM;
        }
        sb->s_root = root;
        return 0;
}

struct super_block *emptyfs_get_sb(struct file_system_type *fs_type,
				  int flags, const char *dev_name, void *data)
{
	/* use the helper function for nodev filesystems */
	return get_sb_nodev(fs_type, flags, data, emptyfs_fill_super);
} 

struct file_system_type emptyfs_fs  = {
	.name = "emptyfs",
	.get_sb = emptyfs_get_sb,
	.kill_sb = kill_litter_super,
};

int __init emptyfs_init(void)
{
	return register_filesystem(&emptyfs_fs);
}
void __exit emptyfs_exit(void)
{
	unregister_filesystem(&emptyfs_fs);
}

module_init(emptyfs_init);
module_exit(emptyfs_exit);
